@@ -33,7 +33,7 @@ const char * MVM_register_type(MVMint8 reg_type) {
 }
 
 static void debug_spill_map(MVMThreadContext *tc, MVMJitCompiler *cl) {
-    MVMint32 i;
+    MVMuint32 i;
     if (!MVM_jit_debug_enabled(tc))
         return;
     MVM_spesh_debug_printf(tc, "JIT Spilled: %d offset %x\n", MVM_VECTOR_ELEMS(cl->spills), cl->spills_base);
@@ -109,6 +109,9 @@ MVMJitCode * MVM_jit_compile_graph(MVMThreadContext *tc, MVMJitGraph *jg) {
         case MVM_JIT_NODE_DATA:
             MVM_jit_emit_data(tc, &cl, &node->u.data);
             break;
+        case MVM_JIT_NODE_DEOPT_CHECK:
+            MVM_jit_emit_deopt_check(tc, &cl);
+            break;
         }
         node = node->next;
     }
@@ -148,7 +151,7 @@ MVMJitCode * MVM_jit_compile_graph(MVMThreadContext *tc, MVMJitGraph *jg) {
 
 MVMJitCode * MVM_jit_compiler_assemble(MVMThreadContext *tc, MVMJitCompiler *cl, MVMJitGraph *jg) {
     MVMJitCode * code;
-    MVMint32 i;
+    MVMuint32 i;
     char * memory;
     size_t codesize;
 
@@ -178,7 +181,7 @@ MVMJitCode * MVM_jit_compiler_assemble(MVMThreadContext *tc, MVMJitCompiler *cl,
     }
 
     /* Create code segment */
-    code = MVM_malloc(sizeof(MVMJitCode));
+    code = MVM_calloc(1, sizeof(MVMJitCode));
 
     code->func_ptr   = (void (*)(MVMThreadContext*,MVMCompUnit*,void*)) memory;
     code->size       = codesize;
@@ -246,7 +249,8 @@ MVMJitCode* MVM_jit_code_copy(MVMThreadContext *tc, MVMJitCode * const code) {
 }
 
 void MVM_jit_code_destroy(MVMThreadContext *tc, MVMJitCode *code) {
-    if (AO_fetch_and_sub1(&code->ref_cnt) > 0)
+    /* fetch_and_sub1 returns previous value, so check if there's only 1 reference */
+    if (AO_fetch_and_sub1(&code->ref_cnt) > 1)
         return;
     MVM_platform_free_pages(code->func_ptr, code->size);
     MVM_free(code->labels);
@@ -307,7 +311,7 @@ void MVM_jit_compile_guard(MVMThreadContext *tc, MVMJitCompiler *compiler,
 void MVM_jit_compile_expr_tree(MVMThreadContext *tc, MVMJitCompiler *compiler, MVMJitGraph *jg, MVMJitExprTree *tree) {
     MVMJitTileList *list;
     MVMJitTile *tile;
-    MVMint32 i;
+    MVMuint32 i;
 
     /* Log what we are planning to compile */
     if (MVM_jit_debug_enabled(tc))
@@ -356,22 +360,23 @@ MVM_STATIC_INLINE MVMint32 reg_type_bucket(MVMint8 reg_type) {
 }
 
 
-MVMint32 MVM_jit_spill_memory_select(MVMThreadContext *tc, MVMJitCompiler *compiler, MVMint8 reg_type) {
-    MVMint32 idx;
+MVMuint32 MVM_jit_spill_memory_select(MVMThreadContext *tc, MVMJitCompiler *compiler, MVMint8 reg_type) {
+    MVMuint32 idx;
     MVMint8 bucket = reg_type_bucket(reg_type);
 
     if (compiler->spills_free[bucket] >= 0) {
         idx = compiler->spills_free[bucket];
         compiler->spills_free[bucket] = compiler->spills[idx].next;
     } else {
-        MVM_VECTOR_ENSURE_SPACE(compiler->spills, idx = compiler->spills_num++);
+        MVM_VECTOR_ENSURE_SPACE(compiler->spills, compiler->spills_num);
+        idx = compiler->spills_num++;
         compiler->spills[idx].reg_type = reg_type;
     }
     return compiler->spills_base + idx * sizeof(MVMRegister);
 }
 
-void MVM_jit_spill_memory_release(MVMThreadContext *tc, MVMJitCompiler *compiler, MVMint32 pos, MVMint8 reg_type) {
-    MVMint32 idx   = (pos - compiler->spills_base) / sizeof(MVMRegister);
+void MVM_jit_spill_memory_release(MVMThreadContext *tc, MVMJitCompiler *compiler, MVMuint32 pos, MVMint8 reg_type) {
+    MVMuint32 idx   = (pos - compiler->spills_base) / sizeof(MVMRegister);
     MVMint8 bucket = reg_type_bucket(reg_type);
     compiler->spills[idx].next    = compiler->spills_free[bucket];
     compiler->spills_free[bucket] = idx;
